"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("./errors");
const assert_1 = require("assert");
exports.SEP_OPERATOR = '__';
exports.SEP_FLAG = '_';
exports.SEP_PATH = '_';
exports.DEFAULT_CONDITION = 'equals';
exports.GET_SEPARATOR = '.';
exports.EVERY_STRING = 'every';
exports.SOME_STRING = 'some';
exports.NONE_STRING = 'none';
exports.and = (values) => {
    return values.filter(returnValue => !returnValue).length === 0;
};
exports.or = (values) => {
    return values.filter(returnValue => returnValue).length > 0;
};
exports.not = (values) => {
    return values.filter(returnValue => returnValue).length === 0;
};
exports.conditions = {
    'every': {
        'alias': ['every'],
        'call': (value) => {
            return exports.and(value);
        },
        'humanlyReadableAs': 'every',
    },
    'some': {
        'alias': ['some'],
        'call': (value) => {
            return exports.or(value);
        },
        'humanlyReadableAs': 'some',
    },
    'none': {
        'alias': ['none'],
        'call': (value) => {
            return exports.not(value);
        },
        'humanlyReadableAs': 'none',
    },
    'startsWith': {
        'alias': ['startsWith', 'sw'],
        'call': (value, expected, flag) => {
            if (typeof value === 'string' && typeof expected === 'string') {
                if (flag.includes('i')) {
                    const tmp = expected.toLowerCase();
                    return tmp.startsWith(value.toLowerCase());
                }
                return expected.startsWith(value);
            }
            return false;
        },
        'humanlyReadableAs': 'starts with',
    },
    'endsWith': {
        'alias': ['endsWith', 'ew'],
        'call': (value, expected, flag) => {
            if (typeof value === 'string' && typeof expected === 'string') {
                if (flag.includes('i')) {
                    const tmp = expected.toLowerCase();
                    return tmp.endsWith(value.toLowerCase());
                }
                return expected.endsWith(value);
            }
            return false;
        },
        'humanlyReadableAs': 'ends with',
    },
    'equals': {
        'alias': ['equals', 'eq'],
        'call': (value, expected, flag) => {
            if (typeof value === typeof expected) {
                if (typeof value === 'object' || typeof expected === 'object') {
                    try {
                        assert_1.strict.deepEqual(value, expected);
                        return true;
                    }
                    catch (_) {
                        return false;
                    }
                }
                if (flag.includes('i')) {
                    return value.toLowerCase() === expected.toLowerCase();
                }
                return value === expected;
            }
            return false;
        },
        'humanlyReadableAs': 'equal',
    },
    'notEquals': {
        'alias': ['ne', 'notEquals'],
        'call': (value, expected, flag) => {
            if (typeof value === 'object' || typeof expected === 'object') {
                try {
                    assert_1.strict.deepEqual(value, expected);
                    return false;
                }
                catch (_) {
                    return true;
                }
            }
            if (flag.includes('i')) {
                return value.toLowerCase() !== expected.toLowerCase();
            }
            return value !== expected;
        },
        'humanlyReadableAs': 'not',
    },
    'contains': {
        'alias': ['contains', 'includes', 'in'],
        'call': (value, expected, flag) => {
            if (typeof value === 'object' && typeof expected === 'object') {
                for (const el in value) {
                    if (expected.hasOwnProperty(el) && (expected[el] !== value[el])) {
                        return false;
                    }
                }
                return true;
            }
            if (typeof value === 'string' && typeof expected === 'string') {
                if (flag.includes('i')) {
                    return expected.toLowerCase().search(value.toLowerCase()) >= 0;
                }
                return expected.search(value) >= 0;
            }
            if (flag.includes('i')) {
                return value.filter((el) => el.toLowerCase()).includes(expected.toLowerCase());
            }
            return value.includes(expected);
        },
        'humanlyReadableAs': 'in',
    },
    'notContains': {
        'alias': ['notContains', 'not_includes', 'nin'],
        'call': (value, expected, flag) => {
            if (typeof value === 'string' && typeof expected === 'string') {
                if (flag.includes('i')) {
                    return expected.toLowerCase().search(value.toLowerCase()) === -1;
                }
                return expected.search(value) === -1;
            }
            if (flag.includes('i')) {
                return !value.filter((el) => el.toLowerCase()).includes(expected.toLowerCase());
            }
            return !value.includes(expected);
        },
        'humanlyReadableAs': 'not in',
    },
    'greaterThan': {
        'alias': ['greaterThan', 'gt'],
        'call': (value, expected, flag) => {
            if (typeof value === 'number' && typeof expected === 'number') {
                return value < expected;
            }
            throw (new errors_1.CheckError({
                'status': false,
                'operator': 'greaterThan',
                'given_value': value,
                'expected': expected,
                'flag': flag,
                'reason': `Error: wrong type: compare ${typeof value} to  ${typeof expected} is not handle`,
            }));
        },
        'humanlyReadableAs': 'greaterThan than',
    },
    'greaterThanEquals': {
        'alias': ['greaterThanEquals', 'gte'],
        'call': (value, expected, flag) => {
            if (typeof value === 'number' && typeof expected === 'number') {
                return value <= expected;
            }
            throw (new errors_1.CheckError({
                'status': false,
                'operator': 'greaterThanEquals',
                'given_value': value,
                'expected': expected,
                'flag': flag,
                'reason': `Error: wrong type: compare ${typeof value} to  ${typeof expected} is not handle`,
            }));
        },
        'humanlyReadableAs': 'greater or equal than',
    },
    'lessThan': {
        'alias': ['lessThan', 'lt'],
        'call': (value, expected, flag) => {
            if (typeof value === 'number' && typeof expected === 'number') {
                return value > expected;
            }
            throw (new errors_1.CheckError({
                'status': false,
                'operator': 'lessThan',
                'given_value': value,
                'expected': expected,
                'flag': flag,
                'reason': `Error: wrong type: compare ${typeof value} to  ${typeof expected} is not handle`,
            }));
        },
        'humanlyReadableAs': 'less than',
    },
    'lessThanEquals': {
        'alias': ['lessequal', 'lte'],
        'call': (value, expected, flag) => {
            if (typeof value === 'number' && typeof expected === 'number') {
                return value >= expected;
            }
            throw (new errors_1.CheckError({
                'status': false,
                'operator': 'lessThanEquals',
                'given_value': value,
                'expected': expected,
                'flag': flag,
                'reason': `Error: wrong type: compare ${typeof value} to  ${typeof expected} is not handle`,
            }));
        },
        'humanlyReadableAs': 'lesser or equal to',
    },
};
